using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using System.Diagnostics;

namespace Foosball
{
    public enum ControlKeys
    {
        Start = 0,   
        A, 
        B,
        Up,
        Down,
        Left,
        Right,
        KEY_LAST,
    };

    public class Control
    {
        private Game _game;
        private PlayerIndex _player;
        private Keys[] _keys;        
        private bool[] _prevState, _justPressed, _justReleased;
        private int _rodBIndex;

        public int RodBIndex
        {
            get
            {
                return _rodBIndex;
            }
            set
            {
                _rodBIndex = value;
            }
        }
        public Control(PlayerIndex playerIndex)
        {
            _rodBIndex = 0;
            _player = playerIndex;
            _keys = new Keys[(int)ControlKeys.KEY_LAST];
            _prevState = new bool[(int)ControlKeys.KEY_LAST];
            _justPressed = new bool[(int)ControlKeys.KEY_LAST];
            _justReleased = new bool[(int)ControlKeys.KEY_LAST];

            if (playerIndex == PlayerIndex.Two)
            {
                _keys[(int)ControlKeys.Start] = Keys.F1;
                _keys[(int)ControlKeys.A] = Keys.NumPad2;
                _keys[(int)ControlKeys.B] = Keys.NumPad3;
                _keys[(int)ControlKeys.Up] = Keys.Up;
                _keys[(int)ControlKeys.Down] = Keys.Down;
                _keys[(int)ControlKeys.Left] = Keys.Left;
                _keys[(int)ControlKeys.Right] = Keys.Right;
            }
            else if (playerIndex == PlayerIndex.One)
            {
                _keys[(int)ControlKeys.Start] = Keys.F2;
                _keys[(int)ControlKeys.A] = Keys.N;
                _keys[(int)ControlKeys.B] = Keys.M;
                _keys[(int)ControlKeys.Up] = Keys.W;
                _keys[(int)ControlKeys.Down] = Keys.S;
                _keys[(int)ControlKeys.Left] = Keys.A;
                _keys[(int)ControlKeys.Right] = Keys.D;
            }
        }

        public bool isPressed(ControlKeys ckey)
        {
            return _game.IsActive && _prevState[(int)ckey];
        }

        public bool isJustPressed(ControlKeys ckey)
        {
            return _game.IsActive && _justPressed[(int)ckey];
        }

        public bool isJustReleased(ControlKeys ckey)
        {
            return _game.IsActive && _justReleased[(int)ckey];
        }

        public void Update(Game game, KeyboardState keyState)
        {
            _game = game;
            for (int i = 0; i < (int)ControlKeys.KEY_LAST; i++)
            {               
                bool curState = keyState.IsKeyDown(_keys[i]);
                if (curState && !_prevState[i])
                    _justPressed[i] = true;
                else
                    _justPressed[i] = false;

                if (!curState && _prevState[i])
                    _justReleased[i] = true;
                else
                    _justReleased[i] = false;
                _prevState[i] = curState;                
            }

            
            if (keyState.IsKeyDown(Keys.D1))
            {
                _rodBIndex = 1;
            }
            if (keyState.IsKeyDown(Keys.D2))
            {
                _rodBIndex = 2;
            }
            if (keyState.IsKeyDown(Keys.D3))
            {
                _rodBIndex = 3;
            }
            if (keyState.IsKeyDown(Keys.D4))
            {
                _rodBIndex = 4;
            }
        }
    }
}
