
#region Using Statements
using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Foosball
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class FoosGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;

        #region Variables and Constants

        public const int X_RESOLUTION = 1024;
        public const int Y_RESOLUTION = 768;

        private int width, height;
        /// <summary>
        /// Foostable dimensions
        /// </summary>
        // Table related
        public const float TABLE_LENGTH     = 222.0F;
        public const float FIELD_LENGTH     = 192.0F;
        public const float FIELD_WIDTH      = 108.0F;
        public const float FIELD_DEPTH      = 17.0F;
        // Rod related
        public const float ROD_TO_SURF      = 12.5F;
        public const float ROD_TO_ROD       = 24.0F;
        public const float ROT_TO_MANFOOT   = 11.75F;
        public const float ROD_TO_WALL      = 12.0F;
        // Ball size
        public const float BALL_SIZE = 5.0F;
        public const float BALL_RADIUS = BALL_SIZE * 0.5F;
        // Gate size
        public const float GATE_WIDTH = 33.5F;
        public const float HALF_GATE_WIDTH = GATE_WIDTH * 0.5F;


        /// <summary>
        /// Models 
        /// </summary>
        private String[] modelName = {
            "Content\\Models\\table",
            "Content\\Models\\table2",
            "Content\\Models\\blackman",
            "Content\\Models\\yellowman",
            "Content\\Models\\ball",
            "Content\\Models\\blackrod",
            "Content\\Models\\yellowrod",
            
        };
        private Model[] modelArray;
        // Model name string indices
        public const int MI_TABLE = 0;
        public const int MI_TABLE2 = 1;
        public const int MI_BLACKMEN = 2;
        public const int MI_YELLOWMEN = 3;
        public const int MI_BALL = 4;
        public const int MI_BLACKROD = 5;
        public const int MI_YELLOWROD = 6;
        

        /// <summary>
        /// FoosObjects
        /// (FoosObj is a wrapping class for Model, multiple FoosObj could share one Model
        /// resource but have different position/orientation)
        /// </summary>
        FoosObj[] allObjs;
        // Objects number
        public const int BLACKMEN_NUM  = 13;
        public const int YELLOWMEN_NUM = 13;
        public const int BLACKROD_NUM = 4;
        public const int YELLOWROD_NUM = 4;
        public const int ALLOBJS_NUM = 
            3 + BLACKMEN_NUM + YELLOWMEN_NUM + BLACKROD_NUM + YELLOWROD_NUM;
        // Objects indices
        public const int OI_TABLE = 0;
        public const int OI_TABLE2 = 1;
        public const int OI_BALL = 2;
        public const int OI_BLACKMEN = 3;
        public const int OI_YELLOWMEN = OI_BLACKMEN + BLACKMEN_NUM;
        public const int OI_BLACKROD = OI_YELLOWMEN + YELLOWMEN_NUM;
        public const int OI_YELLOWROD = OI_BLACKROD + BLACKROD_NUM;

        public const float COOL_DOWN = 5000; // Cool down time for new set (in milliseconds)

        private Control player1Control;
        private Control player2Control;
        private bool justWentFullScreen;

        private static int blackWin = 0;
        private static int yellowWin = 0;
        private static bool newSet = false;
        private static float coolDown;
        private static Random rdm = new Random();

        // Top textures variables
        
        private SpriteBatch ForegroundBatch;

        private String[] spriteName = {
                "Content\\Textures\\player1",        
             //   "Content\\Textures\\player1.jpg",            
            };
        public const int SP_PLAYER1 = 0;
        public const int SP_PLAYER2 = 0;

        private Texture2D[] SpriteTexture;
        private Rectangle[] spriteRect;

        private SpriteFont CourierNew;

        public static float CoolDown
        {
            get
            {
                return coolDown;
            }
            set
            {
                coolDown = value;
            }
        }

        public static int BlackWin
        {
            get
            {
                return blackWin;
            }
            set
            {
                blackWin = value;
            }
        }

        public static int YellowWin
        {
            get
            {
                return yellowWin;
            }
            set
            {
                yellowWin = value;
            }
        }

        public static bool NewSet
        {
            get
            {
                return newSet;
            }
            set
            {
                newSet = value;
            }
        }

        public SoundBank SoundBank
        {
            get
            {
                return soundBank;
            }
        }
        #endregion


       

        public FoosGame()
        {
            content = new ContentManager(Services);            
            graphics = new GraphicsDeviceManager(this);

            graphics.PreferredBackBufferWidth   = X_RESOLUTION;
            graphics.PreferredBackBufferHeight  = Y_RESOLUTION;
            graphics.IsFullScreen = false;
            graphics.ApplyChanges();

            width = graphics.PreferredBackBufferWidth;
            height = graphics.PreferredBackBufferHeight;

            audioEngine = new AudioEngine("Content\\Audio\\foosball.xgs");
            waveBank = new WaveBank(audioEngine, "Content\\Audio\\Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content\\Audio\\Sound Bank.xsb");


            player1Control = new Control(PlayerIndex.One);
            player2Control = new Control(PlayerIndex.Two);
        }       

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here                
            base.Initialize();
        }        

        protected void initModels()
        {
            modelArray = new Model[modelName.Length];
            for (int i = 0; i < modelName.Length; i++)            
                modelArray[i] = content.Load<Model>(modelName[i]);            
        }

        protected void initYellowRodMen()
        {
            // Yellow men's rods
            // 1
            allObjs[OI_YELLOWROD] = new Rod(modelArray[MI_YELLOWROD], 2);
            allObjs[OI_YELLOWROD].positionX = 84.0F;
            allObjs[OI_YELLOWROD].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_YELLOWROD]).Zlimit = 21.0F;
            // 2
            allObjs[OI_YELLOWROD + 1] = new Rod(modelArray[MI_YELLOWROD], 2);
            allObjs[OI_YELLOWROD + 1].positionX = 60.0F;
            allObjs[OI_YELLOWROD + 1].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_YELLOWROD + 1]).Zlimit = 30.0F;
            // 3
            allObjs[OI_YELLOWROD + 2] = new Rod(modelArray[MI_YELLOWROD], 2);
            allObjs[OI_YELLOWROD + 2].positionX = 12.0F;
            allObjs[OI_YELLOWROD + 2].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_YELLOWROD + 2]).Zlimit = 12.0F;
            // 4
            allObjs[OI_YELLOWROD + 3] = new Rod(modelArray[MI_YELLOWROD], 2);
            allObjs[OI_YELLOWROD + 3].positionX = -36.0F;
            allObjs[OI_YELLOWROD + 3].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_YELLOWROD + 3]).Zlimit = 21.0F;


            // Yellow men
            // 0 - 2 are for rod 1
            allObjs[OI_YELLOWMEN] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD], new Vector3(0, 0, 27.0F));
            allObjs[OI_YELLOWMEN].positionX = 84.0F;
            allObjs[OI_YELLOWMEN].positionZ = 27.0F;

            allObjs[OI_YELLOWMEN + 1] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD], new Vector3(0, 0, 0.0F));
            allObjs[OI_YELLOWMEN + 1].positionX = 84.0F;
            allObjs[OI_YELLOWMEN + 1].positionZ = 0.0F;

            allObjs[OI_YELLOWMEN + 2] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD], new Vector3(0, 0, -27.0F));
            allObjs[OI_YELLOWMEN + 2].positionX = 84.0F;
            allObjs[OI_YELLOWMEN + 2].positionZ = -27.0F;

            // 3 - 4 are for rod 2
            allObjs[OI_YELLOWMEN + 3] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 1], new Vector3(0, 0, 18.0F));
            allObjs[OI_YELLOWMEN + 3].positionX = 60.0F;
            allObjs[OI_YELLOWMEN + 3].positionZ = 18.0F;

            allObjs[OI_YELLOWMEN + 4] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 1], new Vector3(0, 0, -18.0F));
            allObjs[OI_YELLOWMEN + 4].positionX = 60.0F;
            allObjs[OI_YELLOWMEN + 4].positionZ = -18.0F;

            // 5 - 9 are for rod 3
            allObjs[OI_YELLOWMEN + 5] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 2], new Vector3(0, 0, 36.0F));
            allObjs[OI_YELLOWMEN + 5].positionX = 12.0F;
            allObjs[OI_YELLOWMEN + 5].positionZ = 36.0F;

            allObjs[OI_YELLOWMEN + 6] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 2], new Vector3(0, 0, 18.0F));
            allObjs[OI_YELLOWMEN + 6].positionX = 12.0F;
            allObjs[OI_YELLOWMEN + 6].positionZ = 18.0F;

            allObjs[OI_YELLOWMEN + 7] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 2], new Vector3(0, 0, 0.0F));
            allObjs[OI_YELLOWMEN + 7].positionX = 12.0F;
            allObjs[OI_YELLOWMEN + 7].positionZ = 0.0F;

            allObjs[OI_YELLOWMEN + 8] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 2], new Vector3(0, 0, -18.0F));
            allObjs[OI_YELLOWMEN + 8].positionX = 12.0F;
            allObjs[OI_YELLOWMEN + 8].positionZ = -18.0F;

            allObjs[OI_YELLOWMEN + 9] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 2], new Vector3(0, 0, -36.0F));
            allObjs[OI_YELLOWMEN + 9].positionX = 12.0F;
            allObjs[OI_YELLOWMEN + 9].positionZ = -36.0F;

            // 10 - 12 are for rod 4
            allObjs[OI_YELLOWMEN + 10] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 3], new Vector3(0, 0, 27.0F));
            allObjs[OI_YELLOWMEN + 10].positionX = -36.0F;
            allObjs[OI_YELLOWMEN + 10].positionZ = 27.0F;

            allObjs[OI_YELLOWMEN + 11] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 3], new Vector3(0, 0, 0.0F));
            allObjs[OI_YELLOWMEN + 11].positionX = -36.0F;
            allObjs[OI_YELLOWMEN + 11].positionZ = 0.0F;

            allObjs[OI_YELLOWMEN + 12] =
                    new FoosMan(modelArray[MI_YELLOWMEN],
                    (Rod)allObjs[OI_YELLOWROD + 3], new Vector3(0, 0, -27.0F));
            allObjs[OI_YELLOWMEN + 12].positionX = -36.0F;
            allObjs[OI_YELLOWMEN + 12].positionZ = -27.0F;
        }

        protected void initBlackRodMen()
        {
            // Black men's rods
            // 1
            allObjs[OI_BLACKROD] = new Rod(modelArray[MI_BLACKROD], 1);
            allObjs[OI_BLACKROD].positionX = -84.0F;
            allObjs[OI_BLACKROD].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_BLACKROD]).Zlimit = 21.0F;
            // 2
            allObjs[OI_BLACKROD + 1] = new Rod(modelArray[MI_BLACKROD], 1);
            allObjs[OI_BLACKROD + 1].positionX = -60.0F;
            allObjs[OI_BLACKROD + 1].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_BLACKROD + 1]).Zlimit = 30.0F;
            // 3
            allObjs[OI_BLACKROD + 2] = new Rod(modelArray[MI_BLACKROD], 1);
            allObjs[OI_BLACKROD + 2].positionX = -12.0F;
            allObjs[OI_BLACKROD + 2].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_BLACKROD + 2]).Zlimit = 12.0F;
            // 4
            allObjs[OI_BLACKROD + 3] = new Rod(modelArray[MI_BLACKROD], 1);
            allObjs[OI_BLACKROD + 3].positionX = 36.0F;
            allObjs[OI_BLACKROD + 3].positionY = ROD_TO_SURF;
            ((Rod)allObjs[OI_BLACKROD + 3]).Zlimit = 21.0F;


            // Black men
            // 0 - 2 are for rod 1
            allObjs[OI_BLACKMEN] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD], new Vector3(0, 0, 27.0F));
            allObjs[OI_BLACKMEN].positionX = -84.0F;
            allObjs[OI_BLACKMEN].positionZ = 27.0F;

            allObjs[OI_BLACKMEN + 1] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD], new Vector3(0, 0, 0.0F));
            allObjs[OI_BLACKMEN + 1].positionX = -84.0F;
            allObjs[OI_BLACKMEN + 1].positionZ = 0.0F;

            allObjs[OI_BLACKMEN + 2] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD], new Vector3(0, 0, -27.0F));
            allObjs[OI_BLACKMEN + 2].positionX = -84.0F;
            allObjs[OI_BLACKMEN + 2].positionZ = -27.0F;

            // 3 - 4 are for rod 2
            allObjs[OI_BLACKMEN + 3] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 1], new Vector3(0, 0, 18.0F));
            allObjs[OI_BLACKMEN + 3].positionX = -60.0F;
            allObjs[OI_BLACKMEN + 3].positionZ = 18.0F;

            allObjs[OI_BLACKMEN + 4] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 1], new Vector3(0, 0, -18.0F));
            allObjs[OI_BLACKMEN + 4].positionX = -60.0F;
            allObjs[OI_BLACKMEN + 4].positionZ = -18.0F;

            // 5 - 9 are for rod 3
            allObjs[OI_BLACKMEN + 5] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 2], new Vector3(0, 0, 36.0F));
            allObjs[OI_BLACKMEN + 5].positionX = -12.0F;
            allObjs[OI_BLACKMEN + 5].positionZ = 36.0F;

            allObjs[OI_BLACKMEN + 6] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 2], new Vector3(0, 0, 18.0F));
            allObjs[OI_BLACKMEN + 6].positionX = -12.0F;
            allObjs[OI_BLACKMEN + 6].positionZ = 18.0F;

            allObjs[OI_BLACKMEN + 7] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 2], new Vector3(0, 0, 0.0F));
            allObjs[OI_BLACKMEN + 7].positionX = -12.0F;
            allObjs[OI_BLACKMEN + 7].positionZ = 0.0F;

            allObjs[OI_BLACKMEN + 8] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 2], new Vector3(0, 0, -18.0F));
            allObjs[OI_BLACKMEN + 8].positionX = -12.0F;
            allObjs[OI_BLACKMEN + 8].positionZ = -18.0F;

            allObjs[OI_BLACKMEN + 9] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 2], new Vector3(0, 0, -36.0F));
            allObjs[OI_BLACKMEN + 9].positionX = -12.0F;
            allObjs[OI_BLACKMEN + 9].positionZ = -36.0F;

            // 10 - 12 are for rod 4
            allObjs[OI_BLACKMEN + 10] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 3], new Vector3(0, 0, 27.0F));
            allObjs[OI_BLACKMEN + 10].positionX = 36.0F;
            allObjs[OI_BLACKMEN + 10].positionZ = 27.0F;

            allObjs[OI_BLACKMEN + 11] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 3], new Vector3(0, 0, 0.0F));
            allObjs[OI_BLACKMEN + 11].positionX = 36.0F;
            allObjs[OI_BLACKMEN + 11].positionZ = 0.0F;

            allObjs[OI_BLACKMEN + 12] =
                    new FoosMan(modelArray[MI_BLACKMEN],
                    (Rod)allObjs[OI_BLACKROD + 3], new Vector3(0, 0, -27.0F));
            allObjs[OI_BLACKMEN + 12].positionX = 36.0F;
            allObjs[OI_BLACKMEN + 12].positionZ = -27.0F;
        }

        protected void initObjs()
        {
            allObjs = new FoosObj[ALLOBJS_NUM];

            // Table
            allObjs[OI_TABLE] = new FoosObj(modelArray[MI_TABLE]);
            allObjs[OI_TABLE2] = new FoosObj(modelArray[MI_TABLE2]);

            // Ball
            allObjs[OI_BALL] = new FBall(modelArray[MI_BALL]);
#if false
            allObjs[OI_BALL].position = new Vector3(70.0F, 2.5F, 14.9F);            
            ((FBall)allObjs[OI_BALL]).Speed = new Vector3(0.02F, 0, 0);
#else
            allObjs[OI_BALL].position = new Vector3(0.0F, 2.5F, 0.0F);            
            newSet = true;
            coolDown = 1000;
#endif

            initBlackRodMen();
            initYellowRodMen();           
        }

        
        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                initModels();
                initObjs();

                ForegroundBatch = new SpriteBatch(graphics.GraphicsDevice);
                SpriteTexture = new Texture2D[spriteName.Length];
                spriteRect = new Rectangle[spriteName.Length];
                for (int i = 0;i < SpriteTexture.Length;i++)
                    SpriteTexture[i] = content.Load<Texture2D>(spriteName[i]);

                calSpriteRect();

                CourierNew = content.Load<SpriteFont>("Content\\couriernew");
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        protected void calSpriteRect()
        {
            double ratioX = width * 1.0 / X_RESOLUTION;
            double ratioY = height * 1.0 / Y_RESOLUTION;

            // Players' labels
            spriteRect[SP_PLAYER1].X = (int)(width * 0.01);            
            spriteRect[SP_PLAYER1].Width = (int)(SpriteTexture[SP_PLAYER1].Width * ratioX);
            spriteRect[SP_PLAYER1].Height = (int)(SpriteTexture[SP_PLAYER1].Height * ratioY);
            spriteRect[SP_PLAYER1].Y = (int)(height * 0.99) - spriteRect[SP_PLAYER1].Height;
            
        }
        

        private void ToggleFullScreen()
        {
            PresentationParameters presentation = graphics.GraphicsDevice.PresentationParameters;

            if (presentation.IsFullScreen)
            {   // going windowed
                graphics.PreferredBackBufferWidth = X_RESOLUTION;
                graphics.PreferredBackBufferHeight = Y_RESOLUTION;
            }
            else
            {
                // going fullscreen, use desktop resolution to minimize display mode changes
                // this also has the nice effect of working around some displays that lie about 
                // supporting 1280x720
                GraphicsAdapter adapter = graphics.GraphicsDevice.CreationParameters.Adapter;
                graphics.PreferredBackBufferWidth = adapter.CurrentDisplayMode.Width;
                graphics.PreferredBackBufferHeight = adapter.CurrentDisplayMode.Height;
            }

            width = graphics.PreferredBackBufferWidth;
            height = graphics.PreferredBackBufferHeight;
            calSpriteRect();

            graphics.ToggleFullScreen();
        }

        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            //if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
            //    this.Exit();           

            // TODO: Add your update logic here
            //allObjs[OI_BLACKMEN].zRotation += (float)gameTime.ElapsedGameTime.TotalMilliseconds * MathHelper.ToRadians(0.02f);
            TimeSpan elapsedTime = gameTime.ElapsedGameTime;
            TimeSpan time = gameTime.TotalGameTime;

            // The time since Update was called last
            float elapsed = (float)elapsedTime.TotalSeconds;
            double elapsedMilliseconds = gameTime.ElapsedGameTime.TotalMilliseconds;
            

            KeyboardState keyState = Keyboard.GetState();
            if (keyState.IsKeyDown(Keys.Escape))
                this.Exit();           


            player1Control.Update(this, keyState);
            player2Control.Update(this, keyState);

            if ((keyState.IsKeyDown(Keys.RightAlt) || keyState.IsKeyDown(Keys.LeftAlt)) && keyState.IsKeyDown(Keys.Enter) && !justWentFullScreen)
            {
                ToggleFullScreen();
                justWentFullScreen = true;
            }

            if (keyState.IsKeyUp(Keys.Enter))
            {
                justWentFullScreen = false;
            }

            /*
            if (XInputHelper.GamePads[PlayerIndex.One].BackPressed ||
                XInputHelper.GamePads[PlayerIndex.Two].BackPressed)
            {
                if (gameState == GameState.PlayEvolved || gameState == GameState.PlayRetro)
                {
                    paused = !paused;
                }

                if (gameState == GameState.LogoSplash)
                {
                    this.Exit();
                }
            }
            
            if (!paused)
            {
                //Update everything
                changeState = currentScreen.Update(time, elapsedTime);

                // Update the AudioEngine - MUST call this every frame!!
                Sound.Update();

                //If either player presses start then reset the game
                if (XInputHelper.GamePads[PlayerIndex.One].StartPressed ||
                    XInputHelper.GamePads[PlayerIndex.Two].StartPressed)
                {
                    changeState = GameState.LogoSplash;
                }

                if (changeState != GameState.None)
                {
                    ChangeState(changeState);
                }
            }*/

            if (newSet)
            {
                FBall ball = (FBall)allObjs[OI_BALL];                
                //if (ball.positionX < -103 || ball.positionX > 103)
                //{
                    coolDown -= (float)elapsedMilliseconds;
                    if (coolDown < 0)
                    {
                        newSet = false;

                        int x = rdm.Next(128, 256);
                        if (rdm.Next(-128, 128) < 0)
                            x = -x;
                        int z = rdm.Next(128, 256);
                        if (rdm.Next(-128, 128) < 0)
                            z = -z;
                        float ll = (float)Math.Sqrt(x * x + z * z);
                        float fx = x / ll;
                        float fz = z / ll;

                        ball.position = new Vector3(0, BALL_RADIUS, 0);
                        ball.Speed = new Vector3(fx * 0.01F, 0, fz * 0.01F);

                    }
                //}
            }

            allObjs[OI_BALL].Update(player1Control, elapsedMilliseconds);            
            for (int i = 0;i < BLACKROD_NUM;i++)
                allObjs[OI_BLACKROD + i].Update(player1Control, elapsedMilliseconds);
            for (int i = 0; i < BLACKMEN_NUM; i++)
                allObjs[OI_BLACKMEN + i].Update(player1Control, elapsedMilliseconds);

            for (int i = 0; i < YELLOWROD_NUM; i++)
                allObjs[OI_YELLOWROD + i].Update(player2Control, elapsedMilliseconds);
            for (int i = 0; i < YELLOWMEN_NUM; i++)
                allObjs[OI_YELLOWMEN + i].Update(player2Control, elapsedMilliseconds);
            
            // Collision detection
            FoosMan.collisionDetection(allObjs, elapsedMilliseconds, this);

            base.Update(gameTime);
        }       

        //Position of the Camera in world space, for our view matrix
        Vector3 cameraPosition = new Vector3(0, 220.0f, 100.0f);
        Vector3 cameraTarget = new Vector3(0, 0, 20);

        //Aspect ratio to use for the projection matrix
        float aspectRatio = X_RESOLUTION * 1.0F / Y_RESOLUTION;

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            
           

            graphics.GraphicsDevice.RenderState.DepthBufferEnable = true;
            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = false;
            graphics.GraphicsDevice.RenderState.AlphaTestEnable = false;
            
            foreach (FoosObj obj in allObjs)
            {
                if (obj == null)
                    continue;

                Model myModel = obj.modelRef;
                if (myModel == null)
                    continue;

                Vector3 Position = obj.position;
                float zRotation  = obj.zRotation;

                //Copy any parent transforms
                Matrix[] transforms = new Matrix[myModel.Bones.Count];
                myModel.CopyAbsoluteBoneTransformsTo(transforms);

                //Draw the model, a model can have multiple meshes, so loop
                foreach (ModelMesh mesh in myModel.Meshes)
                {
                    //This is where the mesh orientation is set, as well as our camera and projection
                    foreach (BasicEffect effect in mesh.Effects)
                    {
                        effect.EnableDefaultLighting();
                        effect.World = transforms[mesh.ParentBone.Index] * Matrix.CreateRotationZ(zRotation)
                            * Matrix.CreateTranslation(Position);
                        effect.View = Matrix.CreateLookAt(cameraPosition, cameraTarget, Vector3.Up);
                        effect.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f),
                            aspectRatio, 1.0f, 10000.0f);
                    }
                    //Draw the mesh, will use the effects set above.
                    mesh.Draw();
                }
            }

            ForegroundBatch.Begin();
           // for (int i = 0;i < SpriteTexture.Length;i++)
             //   ForegroundBatch.Draw(SpriteTexture[i], new Vector2(spriteRect[i].X, spriteRect[i].Y), Color.White);
            String strWinBlack = "PLAYER ONE: " + blackWin.ToString() + " GOALS";
            String strHelpBlack = "Control Men with \'W A S D\'";
            ForegroundBatch.DrawString(CourierNew, strWinBlack, 
                new Vector2(0, 0), Color.LightGreen);
            ForegroundBatch.DrawString(CourierNew, strHelpBlack,
                new Vector2(0, CourierNew.MeasureString(strWinBlack).Y), Color.LightGreen);
            String strWinYellow = "PLAYER TWO: " + yellowWin.ToString() + " GOALS";
            String strHelpYellow = "Control Men with Arrow Keys";
            ForegroundBatch.DrawString(CourierNew, strWinYellow, 
                new Vector2(width - CourierNew.MeasureString(strWinYellow).X, 0), Color.Red);
            ForegroundBatch.DrawString(CourierNew, strHelpYellow, 
                new Vector2(width - CourierNew.MeasureString(strHelpYellow).X, CourierNew.MeasureString(strWinYellow).Y), Color.Red);
            ForegroundBatch.End();
            
            base.Draw(gameTime);
        }
    }
}