using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Foosball
{
    public class Rod : FoosObj
    {
        // Variable to determine directions
        protected enum LRUD { None = 0, Left, Right, Up, Down };
        protected LRUD lorr, uord;
        
        protected float _rot_z_old;     // Old Z rotation
        protected float _accel;         // Current angular speed
        protected float _delta;         // Angular acceleration
        protected float _accel_l;       // Maximum angular spped

        protected float _pos_z_old;     // Old Z position
        protected float _mov_z;         // Current moving speed (up and down)        
        protected float _delta_z;       // Maximum moving speed 
        protected float _pos_z_rl;      // Z axis moving limit

        int _index;

        #region Properties
        public int PlayerIndex
        {
            get
            {
                return _index;
            }
        }

        public float Zlimit
        {            
            set
            {
                _pos_z_rl = value;
            }
        }                
        public float zRotationOld
        {
            get
            {
                return _rot_z_old;
            }
            set
            {
                _rot_z_old = value;
            }
        }
        public float zPosOld
        {
            get
            {
                return _pos_z_old;
            }
            set
            {
                _pos_z_old = value;
            }
        }
        public float rotationSpeed
        {
            get
            {
                return _accel;
            }
            set
            {
                _accel = value;
            }
        }
        public float zMovSpeed
        {
            get
            {
                return _mov_z;
            }
            set
            {
                _mov_z = value;
            }
        }
        #endregion

        public Rod(Model model, int index) : base(model)
        {
            _index = index;

            // Rotation variable initialization
            _rot_z = 0.0F;
            _accel = 0;
            _delta = 0.25F;
            _accel_l = 3 * _delta;

            // Moving variable initialization
            _mov_z = 0;
            _delta_z = 0.1F;
            _pos_z_rl = 0;
        }

        public override void Update(Control control, double milliseconds)
        {
            /*
            if (control.RodBIndex != _index)
            {
                lorr = LRUD.None;
                _accel = 0;
                uord = LRUD.None;
                _mov_z = 0;
                return;
            }*/
            /*****************************************************************/
            // Decide angular speed
            /*****************************************************************/
            // Left & Right
            if (control.isJustPressed(ControlKeys.Left))
            {
                lorr = LRUD.Left;
                _accel = 0;
            }
            else if (control.isJustPressed(ControlKeys.Right))
            {
                lorr = LRUD.Right;
                _accel = 0;
            }

            if (control.isJustReleased(ControlKeys.Left) && lorr == LRUD.Left)
            {
                lorr = LRUD.None;
                _accel = 0;
            }

            if (control.isJustReleased(ControlKeys.Right) && lorr == LRUD.Right)
            {
                lorr = LRUD.None;
                _accel = 0;
            }

            if (control.isPressed(ControlKeys.Left) || control.isPressed(ControlKeys.Right))
                _accel = MathHelper.Min(_accel + _delta, _accel_l);

            // Save old angular rotation
            _rot_z_old = _rot_z;
            switch (lorr)
            {
                case LRUD.Left:
                    _rot_z -= (float)milliseconds * MathHelper.ToRadians(_accel);
                    break;
                case LRUD.Right:
                    _rot_z += (float)milliseconds * MathHelper.ToRadians(_accel);
                    break;
            }

            /*****************************************************************/
            // Decide angular speed
            /*****************************************************************/
            // Up & Down
            if (control.isJustPressed(ControlKeys.Up))
            {
                uord = LRUD.Up;
                _mov_z = _delta_z;
            }
            else if (control.isJustPressed(ControlKeys.Down))
            {
                uord = LRUD.Down;
                _mov_z = _delta_z;
            }

            if (control.isJustReleased(ControlKeys.Up) && uord == LRUD.Up)
            {
                uord = LRUD.None;
                _mov_z = 0;
            }

            if (control.isJustReleased(ControlKeys.Down) && uord == LRUD.Down)
            {
                uord = LRUD.None;
                _mov_z = 0;
            }

            _pos_z_old = _pos.Z;
            switch (uord)
            {
                case LRUD.Up:
                    _pos.Z -= (float)milliseconds * _mov_z;
                    break;
                case LRUD.Down:
                    _pos.Z += (float)milliseconds * _mov_z;
                    break;
            }

            _pos.Z = Math.Max(-_pos_z_rl, _pos.Z);
            _pos.Z = Math.Min(_pos_z_rl, _pos.Z);   
        }
    }
}
